/**************************************************************************
 * tforce.h                                                               *
 *  _                         _                                           *
 * | |_ ___ _ _ ___ ___ ___ _| |___                                       *
 * |  _|  _| | |   |  _| . | . | -_|                                      *
 * |_| |_| |___|_|_|___|___|___|___|                                      *
 *                                                                        *
 * * truncode security development *                                      *
 * http://www.truncode.org                                                *
 * necrose <necrose@truncode.org>                                         *
 *                                                                        *
 * Copyright (c) 2003, truncode                                           *
 * All rights reserved.                                                   *
 *                                                                        *
 * Redistribution and use in source and binary forms, with or without     *
 * modification, are permitted provided that the following conditions are *
 * met: * Redistributions of source code must retain the above copyright  *
 *        notice, this list of conditions and the following disclaimer.   *
 *      * Redistributions in binary form must reproduce the above         *
 *        copyright notice, this list of conditions and the following     *
 *        disclaimer in the documentation and/or other materials provided *
 *        with the distribution.                                          *
 *      * Neither the name of the truncode security development nor the   *
 *        names of its contributors may be used to endorse or promote     *
 *        products derived from this software without specific prior      *
 *        written permission.                                             *
 *                                                                        *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    *
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  *
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   *
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  *
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       *
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  *
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  *
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    *
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  *
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   *
 *                                                                        *
 **************************************************************************/

#ifndef TFORCE_H
#define TFORCE_H

/***************INCLUDES***************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#if defined(LINUX) || defined(BSD) || defined(OS5) || defined(TRU64)
#include <string.h>
#else
#include <strings.h>
#endif
#include <math.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>

#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

/***************DEFINES****************/
#ifndef MAXHOSTNAMELEN
#define MAXHOSTNAMELEN 64
#endif
#define MAXPASSWORDLEN 20

/***************BOOLEANS***************/
/* global booleans */
char DISTRIBUTED;
char NODEID;
char SSNSAVE;
char SSNRESUME;
char FORCEHOST;
char NOCHECK;
char VERBOSE;
char DEBUGGING;

/**************STRUCTURES**************/
struct sockaddr_in rmtaddr;
struct hostent *rmthost;
struct timeval time_bgn;
struct timeval time_end;
struct timeval time_psd;

/* profile structure */
struct profile {
  char *t_host;
  char t_addr[15];
  u_short t_port;
  char *realm;
  char *u_name;
  char *pwd_f;
  u_short c_ret;
  u_short s_time;
  u_long p_ctr;
};

/* session structure */
struct session {
  char *ssn_f;
  char *rsm_f;
  FILE *ssn_fp;
  FILE *rsm_fp;
  FILE *pwd_fp;
  char *t_host;
  char *t_addr;
  int t_port;
  char *realm;
  char *u_name;
  char *pwd_f;
  char *pwd_s;
  u_long p_ctr;
  u_long n_pif;
  u_long n_ppn;
  u_short distbd;
  u_short nodeid;
};

/* working structure */
struct working {
  int sockfd;
  FILE *pwd_fp;
  int c_ctr;
  char *pwd_s;
  char *lgn_s;
  char *enc_s;
  char *out_s;
  char *inp_s;
  u_long p_ctr;
  int itr_s;
  int enc_l;
  u_short t_stat;
};

/* result structure */
struct result {
  char *v_lgn;
  float t_psd;
};

/**************PROTOTYPES**************/
int chkrealm(struct profile *p);
char *httpbf(struct profile *p, struct session *s);
int timeval_subtract (struct timeval *result, struct timeval *x, struct timeval *y);
void twiddle(u_short t_stat);
void usage(char *argv[]);

#endif /* TFORCE_H */
